@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003 Kevin Dougherty <gossip@kdough.net>
+ * Copyright (C) 2003-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,9 +16,12 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Kevin Dougherty <gossip@kdough.net>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <time.h>
 #include <stdlib.h>
 
@@ -34,6 +37,7 @@
 #endif
 
 #include <gdk/gdk.h>
+
 #include "gossip-idle.h"
 
 static time_t timestamp = 0;;
@@ -53,7 +57,7 @@ gossip_idle_get_seconds (void)
 		if (XScreenSaverQueryExtension (GDK_DISPLAY (), &event_base, &error_base)) {
 			ss_info = XScreenSaverAllocInfo ();
 		}
-		
+
 		inited = TRUE;
 	}
 
@@ -66,25 +70,25 @@ gossip_idle_get_seconds (void)
 	if (idle < 3) {
 		return timestamp - time (NULL);
 	}
-	
+
 	timestamp = time (NULL);
 
 	return idle;
-	
+
 #elif defined(HAVE_COCOA)
 	static mach_port_t         port;
 	static io_registry_entry_t object;
 	CFMutableDictionaryRef     properties;
 	CFTypeRef                  idle_object;
 	uint64_t                   idle_time;
-	
+
 	if (!inited) {
 		io_iterator_t iter;
 
 		timestamp = time (NULL);
 
 		inited = TRUE;
-		
+
 		IOMasterPort (MACH_PORT_NULL, &port);
 		IOServiceGetMatchingServices (port,
 					      IOServiceMatching ("IOHIDSystem"),
@@ -96,7 +100,7 @@ gossip_idle_get_seconds (void)
 
 		object = IOIteratorNext (iter);
 	}
-	
+
 	if (!object) {
 		return 5;
 	}
@@ -126,7 +130,7 @@ gossip_idle_get_seconds (void)
 	if (idle_time < 3) {
 		return timestamp - time (NULL);
 	}
-	
+
 	timestamp = time (NULL);
 
 	return idle_time;
