@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2006 Imendio AB
+ * Copyright (C) 2006-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,21 +16,35 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ * Authors: Martyn Russell <martyn@imendio.com>
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <string.h>
+
 #include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <glade/glade.h>
 #include <libnotify/notify.h>
 
+#include <libgossip/gossip-account.h>
+#include <libgossip/gossip-contact.h>
+#include <libgossip/gossip-event.h>
+#include <libgossip/gossip-protocol.h>
+#include <libgossip/gossip-message.h>
+#include <libgossip/gossip-presence.h>
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-conf.h>
+#include <libgossip/gossip-stock.h>
 
+#include "gossip-chat-manager.h"
 #include "gossip-app.h"
 #include "gossip-contact-info-dialog.h"
 #include "gossip-notify.h"
 #include "gossip-preferences.h"
-#include "gossip-stock.h"
+#include "gossip-ui-utils.h"
 
 #define DEBUG_DOMAIN "Notify"
 
@@ -38,13 +52,13 @@
 #define NOTIFY_SUBSCRIPTION_TIME 20000
 
 /* Max length of the body part of a message we show in the
- * notification. 
+ * notification.
  */
-#define NOTIFY_MESSAGE_MAX_LEN 64 
+#define NOTIFY_MESSAGE_MAX_LEN 64
 
 /* Time to wait before we use notification for an account after it has
  * gone online/offline, so we don't spam the sound with online's, etc
- */ 
+ */
 #define NOTIFY_WAIT_TIME 10000
 
 static const gchar *       notify_get_status_from_presence        (GossipPresence     *presence);
@@ -72,7 +86,13 @@ static void                notify_protocol_disconnected_cb        (GossipSession
 								   GossipProtocol     *protocol,
 								   gint                reason,
 								   gpointer            user_data);
-static void                notify_contact_presence_updated_cb     (GossipSession      *session,
+static void                notify_contact_presence_updated_cb     (GossipContact      *contact,
+								   GParamSpec         *param,
+								   gpointer            user_data);
+static void                notify_contact_added_cb                (GossipSession      *session,
+								   GossipContact      *contact,
+								   gpointer            user_data);
+static void                notify_contact_removed_cb              (GossipSession      *session,
 								   GossipContact      *contact,
 								   gpointer            user_data);
 static gboolean            notify_subscription_request_show_cb    (GossipContact      *contact);
@@ -91,12 +111,13 @@ enum {
 	NOTIFY_SHOW_ROSTER
 };
 
-static gboolean    inited = FALSE;
-static GHashTable *account_states = NULL;
-static GHashTable *contact_states = NULL;
-static GHashTable *message_notifications = NULL;
-static GHashTable *event_notifications = NULL;
-static GtkWidget  *attach_widget = NULL;
+static gboolean       inited = FALSE;
+static GHashTable    *account_states = NULL;
+static GHashTable    *contact_states = NULL;
+static GHashTable    *message_notifications = NULL;
+static GHashTable    *event_notifications = NULL;
+static GtkWidget     *attach_widget = NULL;
+static GtkStatusIcon *attach_status_icon = NULL;
 
 static const gchar *
 notify_get_status_from_presence (GossipPresence *presence)
@@ -110,7 +131,7 @@ notify_get_status_from_presence (GossipPresence *presence)
 		state = gossip_presence_get_state (presence);
 		status = gossip_presence_state_get_default_status (state);
 	}
-	
+
 	return status;
 }
 
@@ -135,7 +156,12 @@ static void
 notify_closed_cb (NotifyNotification *notify,
 		  gpointer            user_data)
 {
-	g_object_unref (notify);
+	/* NOTE: We only do this because libnotify<=0.4.3 breaks here
+	 * otherwise, this is all detailed in bug #395588.
+	 */
+#ifndef LIBNOTIFY_BUGGY_CLOSE
+	g_object_unref(notify);
+#endif
 }
 
 static gboolean
@@ -143,11 +169,11 @@ notify_get_is_busy (void)
 {
 	GossipPresence      *presence;
 	GossipPresenceState  state;
-	
-	presence = gossip_session_get_presence (gossip_app_get_session ());
-        state = gossip_presence_get_state (presence);
 
-        if (state == GOSSIP_PRESENCE_STATE_BUSY) {
+	presence = gossip_session_get_presence (gossip_app_get_session ());
+	state = gossip_presence_get_state (presence);
+
+	if (state == GOSSIP_PRESENCE_STATE_BUSY) {
 		return TRUE;
 	}
 
@@ -179,8 +205,8 @@ notify_contact_online (GossipContact *contact)
 	if (notify_get_is_busy ()) {
 		return;
 	}
-	
-	gossip_debug (DEBUG_DOMAIN, "Setting up notification for online contact:'%s'", 
+
+	gossip_debug (DEBUG_DOMAIN, "Setting up notification for online contact:'%s'",
 		      gossip_contact_get_id (contact));
 
 	presence = gossip_contact_get_active_presence (contact);
@@ -191,15 +217,17 @@ notify_contact_online (GossipContact *contact)
 			       &show_avatars);
 
 	if (show_avatars) {
-		pixbuf = gossip_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
+		pixbuf = gossip_avatar_get_pixbuf (gossip_contact_get_avatar (contact));
 	}
 
-	if (!pixbuf) {
+	if (pixbuf) {
+		g_object_ref (pixbuf);
+	} else {
 		/* Fall back to presence state */
-		pixbuf = gossip_pixbuf_for_presence (presence); 
+		pixbuf = gossip_pixbuf_for_presence (presence);
 	}
 
-	title = g_markup_printf_escaped (_("%s has come online"), 
+	title = g_markup_printf_escaped (_("%s has come online"),
 					 gossip_contact_get_name (contact));
 	status = notify_get_status_from_presence (presence);
 
@@ -212,8 +240,8 @@ notify_contact_online (GossipContact *contact)
 			  G_CALLBACK (notify_closed_cb),
 			  NULL);
 
-	if (attach_widget) {
-		notify_notification_attach_to_widget (notify, attach_widget);
+	if (attach_status_icon) {
+		notify_notification_attach_to_status_icon (notify, attach_status_icon);
 	}
 
 	notify_notification_add_action (notify, "default", _("Default"),
@@ -247,18 +275,46 @@ notify_subscription_request_default_cb (NotifyNotification *notify,
 static void
 notify_subscription_request_show (GossipContact *contact)
 {
-	NotifyNotification  *notify;
-	GdkPixbuf           *pixbuf = NULL;
-	GError              *error = NULL;
-	const gchar         *name;
-	gchar               *message;
-	gboolean             show_avatars = FALSE;
+	GossipSession        *session;
+	GossipAccountManager *account_manager;
+	const gchar          *contact_name;
+	guint                 n;
+	NotifyNotification   *notify;
+	GdkPixbuf            *pixbuf = NULL;
+	GError               *error = NULL;
+	gchar                *message;
+	gboolean              show_avatars = FALSE;
 
-	name = gossip_contact_get_name (contact);
-	if (name) {
-		message = g_strdup_printf (_("%s wants to be added to your contact list."), name);
-	} else {
-		message = g_strdup (_("Someone wants to be added to your contact list."));
+	session = gossip_app_get_session ();
+	account_manager = gossip_session_get_account_manager (session);
+	n = gossip_account_manager_get_count (account_manager);
+	contact_name = gossip_contact_get_name (contact);
+
+	if (n > 1) {
+		GossipAccount *account;
+		const gchar   *account_name;
+
+		account = gossip_contact_get_account (contact);
+		account_name = gossip_account_get_name (account);
+
+		if (contact_name) {
+			message = g_strdup_printf (
+				_("%s wants to be added to your contact list for your '%s' account."), 
+				contact_name, account_name);
+		} else {
+			message = g_strdup_printf (
+				_("Someone wants to be added to your contact list for your '%s' account."),
+				account_name);
+		}
+	} else { 
+		if (contact_name) {
+			message = g_strdup_printf (
+				_("%s wants to be added to your contact list."), 
+				contact_name);
+		} else {
+			message = g_strdup_printf (
+				_("Someone wants to be added to your contact list."));
+		}
 	}
 
 	gossip_conf_get_bool (gossip_conf_get (),
@@ -266,21 +322,28 @@ notify_subscription_request_show (GossipContact *contact)
 			      &show_avatars);
 
 	if (show_avatars) {
-		pixbuf = gossip_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
+		pixbuf = gossip_avatar_get_pixbuf (gossip_contact_get_avatar (contact));
 	}
 
-	if (!pixbuf) {
+	if (pixbuf) {
+		g_object_ref (pixbuf);
+	} else {
 		/* Fall back to presence state */
-		pixbuf = gossip_pixbuf_from_stock (GTK_STOCK_DIALOG_QUESTION, 
-						   GTK_ICON_SIZE_MENU);
+		pixbuf = gossip_stock_create_pixbuf (gossip_app_get_window (),
+						     GTK_STOCK_DIALOG_QUESTION,
+						     GTK_ICON_SIZE_MENU);
 	}
 
 	notify = notify_notification_new (_("Subscription request"),
 					  message,
 					  NULL,
-					  attach_widget);
+					  NULL);
 	g_free (message);
 
+	if (attach_status_icon) {
+		notify_notification_attach_to_status_icon (notify, attach_status_icon);
+	}
+
 	notify_notification_set_urgency (notify, NOTIFY_URGENCY_NORMAL);
 	notify_notification_set_timeout (notify, NOTIFY_SUBSCRIPTION_TIME);
 
@@ -320,7 +383,32 @@ notify_new_message_default_cb (NotifyNotification *notify,
 	if (event) {
 		message = GOSSIP_MESSAGE (gossip_event_get_data (event));
 		contact = gossip_message_get_sender (message);
-	
+		
+		gossip_chat_manager_remove_events(gossip_app_get_chat_manager(),contact);
+		
+		g_hash_table_remove (event_notifications, notify);
+		g_hash_table_remove (message_notifications, contact);
+	} else {
+		g_warning ("No event found for NotifyNotification: %p", notify);
+	}
+
+	g_object_unref (event_manager);
+}
+
+static void
+notify_new_message_respond_cb (NotifyNotification *notify,
+			       gchar              *label,
+			       GossipEventManager *event_manager)
+{
+	GossipEvent   *event;
+	GossipMessage *message;
+	GossipContact *contact = NULL;
+
+	event = g_hash_table_lookup (event_notifications, notify);
+	if (event) {
+		message = GOSSIP_MESSAGE (gossip_event_get_data (event));
+		contact = gossip_message_get_sender (message);
+
 		gossip_event_manager_activate (event_manager, event);
 
 		g_hash_table_remove (event_notifications, notify);
@@ -347,8 +435,12 @@ notify_new_message_contact_cb (NotifyNotification *notify,
 
 		message = GOSSIP_MESSAGE (gossip_event_get_data (event));
 		contact = gossip_message_get_sender (message);
-
+		
 		parent = GTK_WINDOW (gossip_app_get_window ());
+		if (!gossip_window_get_is_visible (parent)) {
+			parent = NULL;
+		}
+
 		gossip_contact_info_dialog_show (contact, parent);
 
 		g_hash_table_remove (event_notifications, notify);
@@ -363,7 +455,7 @@ notify_new_message_contact_cb (NotifyNotification *notify,
 static NotifyNotification *
 notify_new_message (GossipEventManager *event_manager,
 		    GossipMessage      *message)
-{ 
+{
 	GossipContact      *contact;
 	NotifyNotification *notify;
 	GdkPixbuf          *pixbuf = NULL;
@@ -379,16 +471,16 @@ notify_new_message (GossipEventManager *event_manager,
 
 	body_copy = g_strdup (gossip_message_get_body (message));
 	body_stripped = g_strstrip (body_copy);
-	
+
 	len = g_utf8_strlen (body_stripped, -1);
 	if (len < 1) {
-		gossip_debug (DEBUG_DOMAIN, "Ignoring new message from:'%s', no message content", 
+		gossip_debug (DEBUG_DOMAIN, "Ignoring new message from:'%s', no message content",
 			      gossip_contact_get_id (contact));
 		g_free (body_copy);
 		return NULL;
 	}
 
-	gossip_debug (DEBUG_DOMAIN, "Setting up notification for new message from:'%s'", 
+	gossip_debug (DEBUG_DOMAIN, "Setting up notification for new message from:'%s'",
 		      gossip_contact_get_id (contact));
 
 	show_avatars = FALSE;
@@ -397,15 +489,19 @@ notify_new_message (GossipEventManager *event_manager,
 			       &show_avatars);
 
 	if (show_avatars) {
-		pixbuf = gossip_pixbuf_avatar_from_contact_scaled (contact, 32, 32);
+		pixbuf = gossip_contact_get_avatar_pixbuf (contact);
 	}
 
-	if (!pixbuf) {
+	if (pixbuf) {
+		g_object_ref (pixbuf);
+	} else {
 		/* Fall back to message icon */
-		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE, GTK_ICON_SIZE_MENU);
+		pixbuf = gossip_stock_create_pixbuf (gossip_app_get_window (),
+						     GOSSIP_STOCK_MESSAGE, 
+						     GTK_ICON_SIZE_MENU);
 	}
 
-	title = g_strdup_printf (_("New message from %s"), 
+	title = g_strdup_printf (_("New message from %s"),
 				 gossip_contact_get_name (contact));
 
 	if (len >= NOTIFY_MESSAGE_MAX_LEN) {
@@ -413,7 +509,7 @@ notify_new_message (GossipEventManager *event_manager,
 
 		end = g_utf8_offset_to_pointer (body_stripped, NOTIFY_MESSAGE_MAX_LEN);
 		end[0] = '\0';
-		
+
 		str = g_markup_printf_escaped ("%s...", body_stripped);
 	} else {
 		str = g_markup_printf_escaped ("%s", body_stripped);
@@ -429,15 +525,15 @@ notify_new_message (GossipEventManager *event_manager,
 			  G_CALLBACK (notify_closed_cb),
 			  NULL);
 
-	if (attach_widget) {
-		notify_notification_attach_to_widget (notify, attach_widget);
+	if (attach_status_icon) {
+		notify_notification_attach_to_status_icon (notify, attach_status_icon);
 	}
-	
+
 	notify_notification_add_action (notify, "default", _("Default"),
 					(NotifyActionCallback) notify_new_message_default_cb,
 					g_object_ref (event_manager), NULL);
 	notify_notification_add_action (notify, "respond", _("Show"),
-					(NotifyActionCallback) notify_new_message_default_cb,
+					(NotifyActionCallback) notify_new_message_respond_cb,
 					g_object_ref (event_manager), NULL);
 
 	if (gossip_contact_get_type (contact) == GOSSIP_CONTACT_TYPE_TEMPORARY) {
@@ -446,7 +542,7 @@ notify_new_message (GossipEventManager *event_manager,
 			(NotifyActionCallback) notify_new_message_contact_cb,
 			g_object_ref (event_manager), NULL);
 	}
-	
+
 	if (!notify_notification_show (notify, &error)) {
 		g_warning ("Failed to send notification: %s",
 			   error->message);
@@ -480,7 +576,7 @@ notify_disconnected_message_foreach (GossipContact  *contact,
 	GossipAccount *contact_account;
 
 	contact_account = gossip_contact_get_account (contact);
-	
+
 	if (gossip_account_equal (contact_account, account)) {
 		return TRUE;
 	}
@@ -496,7 +592,7 @@ notify_disconnected_contact_foreach (GossipContact  *contact,
 	GossipAccount *contact_account;
 
 	contact_account = gossip_contact_get_account (contact);
-	
+
 	if (gossip_account_equal (contact_account, account)) {
 		return TRUE;
 	}
@@ -512,11 +608,11 @@ notify_protocol_disconnected_cb (GossipSession  *session,
 				 gpointer        user_data)
 {
 	g_hash_table_remove (account_states, account);
-	
+
 	g_hash_table_foreach_remove (message_notifications,
 				     (GHRFunc) notify_disconnected_message_foreach,
 				     account);
-	
+
 	g_hash_table_foreach_remove (contact_states,
 				     (GHRFunc) notify_disconnected_contact_foreach,
 				     account);
@@ -528,24 +624,27 @@ notify_protocol_connected_cb (GossipSession  *session,
 			      GossipProtocol *protocol,
 			      gpointer        user_data)
 {
-	guint id;
+	guint        id;
+	const gchar *account_name;
 
 	if (g_hash_table_lookup (account_states, account)) {
 		return;
 	}
 
+	account_name = gossip_account_get_name (account);
 	gossip_debug (DEBUG_DOMAIN, "Account update, account:'%s' is now online",
-		      gossip_account_get_id (account));
+		      account_name);
 
 	id = g_timeout_add (NOTIFY_WAIT_TIME,
-			    (GSourceFunc) notify_protocol_timeout_cb, 
+			    (GSourceFunc) notify_protocol_timeout_cb,
 			    account);
-	g_hash_table_insert (account_states, account, GUINT_TO_POINTER (id));
+	g_hash_table_insert (account_states, g_object_ref (account),
+			     GUINT_TO_POINTER (id));
 }
 
 static void
-notify_contact_presence_updated_cb (GossipSession *session,
-				    GossipContact *contact,
+notify_contact_presence_updated_cb (GossipContact *contact,
+				    GParamSpec    *param,
 				    gpointer       user_data)
 {
 	GossipPresence *presence;
@@ -560,30 +659,55 @@ notify_contact_presence_updated_cb (GossipSession *session,
 			gossip_debug (DEBUG_DOMAIN, "Presence update, contact:'%s' is now offline",
 				      gossip_contact_get_id (contact));
 		}
-			
+
 		g_hash_table_remove (contact_states, contact);
 	} else {
 		GossipAccount *account;
-		
+
 		account = gossip_contact_get_account (contact);
 
 		/* Only show notifications after being online for some
 		 * time instead of spamming notifications each time we
 		 * connect.
 		 */
-		if (!g_hash_table_lookup (account_states, account) && 
+		if (!g_hash_table_lookup (account_states, account) &&
 		    !g_hash_table_lookup (contact_states, contact)) {
 			gossip_debug (DEBUG_DOMAIN, "Presence update, contact:'%s' is now online",
 				      gossip_contact_get_id (contact));
 			notify_contact_online (contact);
 		}
-		
-		g_hash_table_insert (contact_states, 
-				     g_object_ref (contact), 
+
+		g_hash_table_insert (contact_states,
+				     g_object_ref (contact),
 				     g_object_ref (presence));
 	}
 }
 
+static void
+notify_contact_added_cb (GossipSession *session,
+			 GossipContact *contact,
+			 gpointer       user_data)
+{
+	g_signal_connect (contact, "notify::presences",
+			  G_CALLBACK (notify_contact_presence_updated_cb),
+			  NULL);
+	g_signal_connect (contact, "notify::type",
+			  G_CALLBACK (notify_contact_presence_updated_cb),
+			  NULL);
+}
+
+static void
+notify_contact_removed_cb (GossipSession *session,
+			   GossipContact *contact,
+			   gpointer       user_data)
+{
+	g_signal_handlers_disconnect_by_func (contact,
+					      notify_contact_presence_updated_cb,
+					      NULL);
+
+	g_hash_table_remove (contact_states, contact);
+}
+
 static gboolean
 notify_subscription_request_show_cb (GossipContact *contact)
 {
@@ -598,7 +722,7 @@ notify_event_added_cb (GossipEventManager *event_manager,
 		       gpointer            user_data)
 {
 	GossipEventType type;
-	
+
 	type = gossip_event_get_type (event);
 
 	if (type == GOSSIP_EVENT_NEW_MESSAGE) {
@@ -614,14 +738,14 @@ notify_event_added_cb (GossipEventManager *event_manager,
 		 */
 		if (!g_hash_table_lookup (message_notifications, contact)) {
 			notify = notify_new_message (event_manager, message);
-			
+
 			if (notify) {
 				g_hash_table_insert (message_notifications,
 						     contact,
 						     g_object_ref (event));
-				g_hash_table_insert (event_notifications,   
+				g_hash_table_insert (event_notifications,
 						     g_object_ref (notify),
-						     g_object_ref (event));  
+						     g_object_ref (event));
 			}
 		}
 	}
@@ -641,7 +765,7 @@ notify_message_remove_foreach (gpointer     key,
 	if (gossip_event_equal (event, event_to_compare)) {
 		return TRUE;
 	}
-	
+
 	return FALSE;
 }
 
@@ -654,7 +778,7 @@ notify_event_remove_foreach (gpointer     key,
 		notify_notification_close (key, NULL);
 		return TRUE;
 	}
-	
+
 	return FALSE;
 }
 
@@ -663,10 +787,10 @@ notify_event_removed_cb (GossipEventManager *event_manager,
 			 GossipEvent        *event,
 			 gpointer            user_data)
 {
-	g_hash_table_foreach_remove (message_notifications, 
+	g_hash_table_foreach_remove (message_notifications,
 				     (GHRFunc) notify_message_remove_foreach,
 				     event);
-	g_hash_table_foreach_remove (event_notifications, 
+	g_hash_table_foreach_remove (event_notifications,
 				     (GHRFunc) notify_event_remove_foreach,
 				     event);
 }
@@ -678,7 +802,7 @@ gossip_notify_set_attach_widget (GtkWidget *new_attach_widget)
 		g_object_remove_weak_pointer (G_OBJECT (attach_widget),
 					      (gpointer) &attach_widget);
 	}
-	
+
 	attach_widget = new_attach_widget;
 	if (attach_widget) {
 		g_object_add_weak_pointer (G_OBJECT (new_attach_widget),
@@ -686,6 +810,21 @@ gossip_notify_set_attach_widget (GtkWidget *new_attach_widget)
 	}
 }
 
+void
+gossip_notify_set_attach_status_icon (GtkStatusIcon *new_attach)
+{
+	if (attach_status_icon) {
+		g_object_remove_weak_pointer (G_OBJECT (attach_status_icon),
+					      (gpointer) &attach_status_icon);
+	}
+
+	attach_status_icon = new_attach;
+	if (attach_status_icon) {
+		g_object_add_weak_pointer (G_OBJECT (attach_status_icon),
+					   (gpointer) &attach_status_icon);
+	}
+}
+
 static void
 notify_hint_closed_cb (NotifyNotification *notification,
 		       gpointer            user_data)
@@ -706,7 +845,7 @@ notify_hint_closed_cb (NotifyNotification *notification,
 }
 
 gboolean
-gossip_notify_hint_show (const gchar        *conf_path, 
+gossip_notify_hint_show (const gchar        *conf_path,
 			 const gchar        *summary,
 			 const gchar        *message,
 			 GFunc               func,
@@ -720,15 +859,15 @@ gossip_notify_hint_show (const gchar        *conf_path,
 	g_return_val_if_fail (conf_path != NULL, FALSE);
 	g_return_val_if_fail (summary != NULL, FALSE);
 
-	ok = gossip_conf_get_bool (gossip_conf_get (), 
-				   conf_path, 
+	ok = gossip_conf_get_bool (gossip_conf_get (),
+				   conf_path,
 				   &show_hint);
-	
+
 	if (ok && !show_hint) {
 		return FALSE;
 	}
 
-	notify = notify_notification_new (summary, message, NULL, attach_widget);
+	notify = notify_notification_new (summary, message, NULL, NULL);
 	g_object_set_data_full (G_OBJECT (notify), "conf_path", g_strdup (conf_path), g_free);
 	g_object_set_data (G_OBJECT (notify), "func", func);
 	g_signal_connect (notify,
@@ -736,6 +875,10 @@ gossip_notify_hint_show (const gchar        *conf_path,
 			  G_CALLBACK (notify_hint_closed_cb),
 			  user_data);
 
+	if (attach_status_icon) {
+		notify_notification_attach_to_status_icon (notify, attach_status_icon);
+	}
+
 	if (!notify_notification_show (notify, &error)) {
 		g_warning ("Failed to send notification: %s",
 			   error->message);
@@ -751,13 +894,13 @@ gossip_notify_init (GossipSession      *session,
 {
 	g_return_if_fail (GOSSIP_IS_SESSION (session));
 	g_return_if_fail (GOSSIP_IS_EVENT_MANAGER (event_manager));
-	
+
 	gossip_debug (DEBUG_DOMAIN, "Initiating...");
 
 	if (inited) {
 		return;
 	}
-	
+
 	if (!notify_init (PACKAGE_NAME)) {
 		g_warning ("Cannot initialize Notify integration");
 		return;
@@ -789,8 +932,11 @@ gossip_notify_init (GossipSession      *session,
 	g_signal_connect (session, "protocol-disconnected",
 			  G_CALLBACK (notify_protocol_disconnected_cb),
 			  NULL);
-	g_signal_connect (session, "contact-presence-updated",
-			  G_CALLBACK (notify_contact_presence_updated_cb),
+	g_signal_connect (session, "contact-added",
+			  G_CALLBACK (notify_contact_added_cb),
+			  NULL);
+	g_signal_connect (session, "contact-removed",
+			  G_CALLBACK (notify_contact_removed_cb),
 			  NULL);
 
 	g_signal_connect (event_manager, "event-added",
@@ -800,7 +946,7 @@ gossip_notify_init (GossipSession      *session,
 			  G_CALLBACK (notify_event_removed_cb),
 			  NULL);
 
- 	inited = TRUE;
+	inited = TRUE;
 }
 
 void
@@ -814,6 +960,6 @@ gossip_notify_finalize (void)
 	g_hash_table_destroy (contact_states);
 	g_hash_table_destroy (message_notifications);
 	g_hash_table_destroy (event_notifications);
-	
+
 	notify_uninit ();
 }
