@@ -1,7 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2004 Martyn Russell <mr@gnome.org>
- * Copyright (C) 2006 Imendio AB
+ * Copyright (C) 2004-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,16 +16,14 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Authors: Martyn Russell <martyn@imendio.com>
+ *          Richard Hult <richard@imendio.com>
  */
 
 #ifndef __GOSSIP_SPELL_DIALOG_H__
 #define __GOSSIP_SPELL_DIALOG_H__
 
-#include <glib.h>
-#include <gtk/gtktextiter.h>
-
-#include "gossip-chat.h"
-
 G_BEGIN_DECLS
 
 void gossip_spell_dialog_show (GossipChat  *chat,
