@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2006 Imendio AB
+ * Copyright (C) 2002-2007 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,15 +16,16 @@
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ * 
+ *          Part of this file is copied from GtkSourceView (gtksourceiter.c):
+ *          Paolo Maggi
+ *          Jeroen Zwartepoorte
  */
 
-/* Part of this file is copied from GtkSourceView (gtksourceiter.c):
- *  Copyright (C) 2000 - 2005 Paolo Maggi
- *  Copyright (C) 2002, 2003 Jeroen Zwartepoorte
- */
+#include "config.h"
 
-#include <config.h>
 #include <string.h>
+
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
@@ -37,181 +38,19 @@
 #include <Cocoa/Cocoa.h>
 #endif
 
+#include <libgossip/gossip-session.h>
+#include <libgossip/gossip-account-manager.h>
+#include <libgossip/gossip-conf.h>
+#include <libgossip/gossip-paths.h>
+#include <libgossip/gossip-stock.h>
+
 #ifdef HAVE_LIBNOTIFY
 #include "gossip-notify.h"
 #endif
 
-#include <libgossip/gossip-conf.h>
-#include <libgossip/gossip-paths.h>
-
-#include "gossip-stock.h"
 #include "gossip-app.h"
 #include "gossip-ui-utils.h"
 
-struct SizeData {
-	gint     width;
-	gint     height;
-	gboolean preserve_aspect_ratio;
-};
-
-static void hint_dialog_response_cb             (GtkWidget       *widget,
-						 gint             response,
-						 GtkWidget       *checkbutton);
-static void pixbuf_from_avatar_size_prepared_cb (GdkPixbufLoader *loader,
-						 gint             width,
-						 gint             height,
-						 struct SizeData *data);
-
-static GladeXML *
-get_glade_file (const gchar *filename,
-		const gchar *root,
-		const gchar *domain,
-		const gchar *first_required_widget,
-		va_list      args)
-{
-	gchar      *path;
-	GladeXML   *gui;
-	const char *name;
-	GtkWidget **widget_ptr;
-
-	path = gossip_paths_get_glade_path (filename);
-	gui = glade_xml_new (path, root, domain);
-	g_free (path);
-
-	if (!gui) {
-		g_warning ("Couldn't find necessary glade file '%s'", filename);
-		return NULL;
-	}
-
-	for (name = first_required_widget; name; name = va_arg (args, char *)) {
-		widget_ptr = va_arg (args, void *);
-
-		*widget_ptr = glade_xml_get_widget (gui, name);
-
-		if (!*widget_ptr) {
-			g_warning ("Glade file '%s' is missing widget '%s'.",
-				   filename, name);
-			continue;
-		}
-	}
-
-	return gui;
-}
-
-void
-gossip_glade_get_file_simple (const gchar *filename,
-			      const gchar *root,
-			      const gchar *domain,
-			      const gchar *first_required_widget, ...)
-{
-	va_list   args;
-	GladeXML *gui;
-
-	va_start (args, first_required_widget);
-
-	gui = get_glade_file (filename,
-			      root,
-			      domain,
-			      first_required_widget,
-			      args);
-
-	va_end (args);
-
-	if (!gui) {
-		return;
-	}
-
-	g_object_unref (gui);
-}
-
-GladeXML *
-gossip_glade_get_file (const gchar *filename,
-		       const gchar *root,
-		       const gchar *domain,
-		       const gchar *first_required_widget, ...)
-{
-	va_list   args;
-	GladeXML *gui;
-
-	va_start (args, first_required_widget);
-
-	gui = get_glade_file (filename,
-			      root,
-			      domain,
-			      first_required_widget,
-			      args);
-
-	va_end (args);
-
-	if (!gui) {
-		return NULL;
-	}
-
-	return gui;
-}
-
-void
-gossip_glade_connect (GladeXML *gui,
-		      gpointer  user_data,
-		      gchar     *first_widget, ...)
-{
-	va_list      args;
-	const gchar *name;
-	const gchar *signal;
-	GtkWidget   *widget;
-	gpointer    *callback;
-
-	va_start (args, first_widget);
-
-	for (name = first_widget; name; name = va_arg (args, char *)) {
-		signal = va_arg (args, void *);
-		callback = va_arg (args, void *);
-
-		widget = glade_xml_get_widget (gui, name);
-		if (!widget) {
-			g_warning ("Glade file is missing widget '%s', aborting",
-				   name);
-			continue;
-		}
-
-		g_signal_connect (widget,
-				  signal,
-				  G_CALLBACK (callback),
-				  user_data);
-	}
-
-	va_end (args);
-}
-
-void
-gossip_glade_setup_size_group (GladeXML         *gui,
-			       GtkSizeGroupMode  mode,
-			       gchar            *first_widget, ...)
-{
-	va_list       args;
-	GtkWidget    *widget;
-	GtkSizeGroup *size_group;
-	const gchar  *name;
-
-	va_start (args, first_widget);
-
-	size_group = gtk_size_group_new (mode);
-
-	for (name = first_widget; name; name = va_arg (args, char *)) {
-		widget = glade_xml_get_widget (gui, name);
-		if (!widget) {
-			g_warning ("Glade file is missing widget '%s'", name);
-			continue;
-		}
-
-		gtk_size_group_add_widget (size_group, widget);
-	}
-
-	g_object_unref (size_group);
-
-	va_end (args);
-}
-
 static void
 password_dialog_activate_cb (GtkWidget *entry, GtkDialog *dialog)
 {
@@ -228,7 +67,7 @@ gossip_password_dialog_run (GossipAccount *account,
 	GtkWidget   *vbox;
 	gchar       *str;
 	gchar       *password;
-	const gchar *id;
+	const gchar *name;
 
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
 
@@ -242,8 +81,8 @@ gossip_password_dialog_run (GossipAccount *account,
 						     str);
 	g_free (str);
 
-	gossip_account_param_get (account, "id", &id, NULL);
-	str = g_strdup_printf (_("Logging in with: %s"), id);
+	name = gossip_account_get_name (account);
+	str = g_strdup_printf (_("Logging in to account '%s'"), name);
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", str);
 	g_free (str);
 
@@ -278,7 +117,7 @@ gossip_password_dialog_run (GossipAccount *account,
 			session = gossip_app_get_session ();
 			manager = gossip_session_get_account_manager (session);
 
-			gossip_account_param_set (account, "password", password, NULL);
+			gossip_account_set_param (account, "password", password, NULL);
 			gossip_account_manager_store (manager);
 		}
 	} else {
@@ -290,244 +129,6 @@ gossip_password_dialog_run (GossipAccount *account,
 	return password;
 }
 
-static void
-hint_dialog_response_cb (GtkWidget *widget,
-			 gint       response,
-			 GtkWidget *checkbutton)
-{
-	GFunc        func;
-	gpointer     user_data;
-	const gchar *conf_path;
-	gboolean     hide_hint;
-
-	conf_path = g_object_get_data (G_OBJECT (widget), "conf_path");
-	func = g_object_get_data (G_OBJECT (widget), "func");
-	user_data = g_object_get_data (G_OBJECT (widget), "user_data");
-
-	hide_hint = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton));
-	gossip_conf_set_bool (gossip_conf_get (), conf_path, !hide_hint);
-
-	gtk_widget_destroy (widget);
-
-	if (func) {
-		(func) ((gpointer) conf_path, user_data);
-	}
-}
-
-gboolean
-gossip_hint_dialog_show (const gchar *conf_path,
-			 const gchar *message1,
-			 const gchar *message2,
-			 GtkWindow   *parent,
-			 GFunc        func,
-			 gpointer     user_data)
-{
-	GtkWidget *dialog;
-	GtkWidget *checkbutton;
-	GtkWidget *vbox;
-	gboolean   ok;
-	gboolean   show_hint = TRUE;
-
-	g_return_val_if_fail (conf_path != NULL, FALSE);
-	g_return_val_if_fail (message1 != NULL, FALSE);
-
-	ok = gossip_conf_get_bool (gossip_conf_get (),
-				   conf_path,
-				   &show_hint);
-
-	if (ok && !show_hint) {
-		return FALSE;
-	}
-
-	dialog = gtk_message_dialog_new_with_markup (parent,
-						     GTK_DIALOG_DESTROY_WITH_PARENT,
-						     GTK_MESSAGE_INFO,
-						     GTK_BUTTONS_CLOSE,
-						     "<b>%s</b>",
-						     message1);
-
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  "%s", message2);
-	checkbutton = gtk_check_button_new_with_label (_("Do not show this again"));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton), TRUE);
-
-	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_container_set_border_width  (GTK_CONTAINER (vbox), 6);
-	gtk_box_pack_start (GTK_BOX (vbox), checkbutton, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox, FALSE, FALSE, 0);
-
-	g_object_set_data_full (G_OBJECT (dialog), "conf_path", g_strdup (conf_path), g_free);
-	g_object_set_data (G_OBJECT (dialog), "user_data", user_data);
-	g_object_set_data (G_OBJECT (dialog), "func", func);
-
-	g_signal_connect (dialog, "response",
-			  G_CALLBACK (hint_dialog_response_cb),
-			  checkbutton);
-
-	gtk_widget_show_all (dialog);
-
-	return TRUE;
-}
-
-gboolean
-gossip_hint_show (const gchar         *conf_path,
-		  const gchar         *message1,
-		  const gchar         *message2,
-		  GtkWindow           *parent,
-		  GFunc                func,
-		  gpointer             user_data)
-{
-#ifdef HAVE_LIBNOTIFY
-	return gossip_notify_hint_show (conf_path,
-					message1, message2,
-					func, user_data);
-#else
-	return gossip_hint_dialog_show (conf_path,
-					message1, message2,
-					parent,
-					func, user_data);
-#endif
-}
-
-GdkPixbuf *
-gossip_pixbuf_from_stock (const gchar *stock,
-			  GtkIconSize  size)
-{
-	return gtk_widget_render_icon (gossip_app_get_window (),
-				       stock, size, NULL);
-}
-
-GdkPixbuf *
-gossip_pixbuf_from_account_type (GossipAccountType type,
-				 GtkIconSize       icon_size)
-{
-	GtkIconTheme  *theme;
-	GdkPixbuf     *pixbuf = NULL;
-	GError        *error = NULL;
-	gint           w, h;
-	gint           size = 48;
-	const gchar   *icon_id = NULL;
-
-	theme = gtk_icon_theme_get_default ();
-
-	if (!gtk_icon_size_lookup (icon_size, &w, &h)) {
-		size = 48;
-	} else {
-		size = (w + h) / 2;
-	}
-
-	switch (type) {
-	case GOSSIP_ACCOUNT_TYPE_JABBER:
-		icon_id = "im-jabber";
-		break;
-	case GOSSIP_ACCOUNT_TYPE_AIM:
-		icon_id = "im-aim";
-		break;
-	case GOSSIP_ACCOUNT_TYPE_ICQ:
-		icon_id = "im-icq";
-		break;
-	case GOSSIP_ACCOUNT_TYPE_MSN:
-		icon_id = "im-msn";
-		break;
-	case GOSSIP_ACCOUNT_TYPE_YAHOO:
-		icon_id = "im-yahoo";
-		break;
-	default:
-		g_assert_not_reached ();
-	}
-
-	pixbuf = gtk_icon_theme_load_icon (theme,
-					   icon_id,     /* icon name */
-					   size,        /* size */
-					   0,           /* flags */
-					   &error);
-
-	return pixbuf;
-}
-
-GdkPixbuf *
-gossip_pixbuf_from_account (GossipAccount *account,
-			    GtkIconSize    icon_size)
-{
-	GossipAccountType  type;
-	GdkPixbuf         *pixbuf = NULL;
-
-	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
-
-	type = gossip_account_get_type (account);
-	pixbuf = gossip_pixbuf_from_account_type (type, icon_size);
-
-	return pixbuf;
-}
-
-GdkPixbuf *
-gossip_pixbuf_from_account_status (GossipAccount *account,
-				   GtkIconSize    icon_size,
-				   gboolean       online)
-{
-	GdkPixbuf *pixbuf = NULL;
-
-	g_return_val_if_fail (GOSSIP_IS_ACCOUNT (account), NULL);
-
-	pixbuf = gossip_pixbuf_from_account (account, icon_size);
-	g_return_val_if_fail (pixbuf != NULL, NULL);
-
-	if (!online) {
-		GdkPixbuf *modded_pixbuf;
-
-		modded_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-						TRUE,
-						8,
-						gdk_pixbuf_get_width (pixbuf),
-						gdk_pixbuf_get_height (pixbuf));
-
-		gdk_pixbuf_saturate_and_pixelate (pixbuf,
-						  modded_pixbuf,
-						  1.0,
-						  TRUE);
-		g_object_unref (pixbuf);
-		pixbuf = modded_pixbuf;
-	}
-
-	return pixbuf;
-}
-
-GdkPixbuf *
-gossip_pixbuf_from_account_error (GossipAccount *account,
-				  GtkIconSize    icon_size)
-{
-	GdkPixbuf *pixbuf;
-	GdkPixbuf *pixbuf_error;
-
-	pixbuf = gossip_pixbuf_from_account_status (account,
-						    icon_size,
-						    FALSE);
-	if (!pixbuf) {
-		return NULL;
-	}
-
-	pixbuf_error = gossip_pixbuf_from_stock (GTK_STOCK_DIALOG_ERROR,
-						 GTK_ICON_SIZE_MENU);
-	if (!pixbuf_error) {
-		g_object_unref (pixbuf);
-		return NULL;
-	}
-
-	gdk_pixbuf_composite (pixbuf_error,
-			      pixbuf,
-			      0, 0,
-			      gdk_pixbuf_get_width (pixbuf_error),
-			      gdk_pixbuf_get_height (pixbuf_error),
-			      0, 0,
-			      1, 1,
-			      GDK_INTERP_BILINEAR,
-			      255);
-
-	g_object_unref (pixbuf_error);
-
-	return pixbuf;
-}
-
 GdkPixbuf *
 gossip_pixbuf_from_smiley (GossipSmiley type,
 			   GtkIconSize  icon_size)
@@ -647,8 +248,9 @@ gossip_pixbuf_from_smiley (GossipSmiley type,
 GdkPixbuf *
 gossip_pixbuf_offline (void)
 {
-	return gossip_pixbuf_from_stock (GOSSIP_STOCK_OFFLINE,
-					 GTK_ICON_SIZE_MENU);
+	return gossip_stock_create_pixbuf (gossip_app_get_window (),
+					   GOSSIP_STOCK_OFFLINE,
+					   GTK_ICON_SIZE_MENU);
 }
 
 static gboolean
@@ -671,6 +273,8 @@ gossip_window_get_is_visible (GtkWindow *window)
 {
 	gboolean visible;
 
+	g_return_val_if_fail (window != NULL, FALSE);
+
 	g_object_get (window,
 		      "visible", &visible,
 		      NULL);
@@ -687,6 +291,8 @@ gossip_window_present (GtkWindow *window,
 	gboolean on_current;
 	guint32  timestamp;
 
+	g_return_if_fail (window != NULL);
+
 	/* There are three cases: hidden, visible, visible on another
 	 * workspace.
 	 */
@@ -734,7 +340,8 @@ gossip_pixbuf_for_presence_state (GossipPresenceState state)
 		break;
 	}
 
-	return gossip_pixbuf_from_stock (stock, GTK_ICON_SIZE_MENU);
+	return gossip_stock_create_pixbuf (gossip_app_get_window (),
+					   stock, GTK_ICON_SIZE_MENU);
 }
 
 GdkPixbuf *
@@ -769,8 +376,9 @@ gossip_pixbuf_for_contact (GossipContact *contact)
 
 	if (subscription != GOSSIP_SUBSCRIPTION_BOTH &&
 	    subscription != GOSSIP_SUBSCRIPTION_TO) {
-		return gossip_pixbuf_from_stock (GOSSIP_STOCK_PENDING,
-						 GTK_ICON_SIZE_MENU);
+		return gossip_stock_create_pixbuf (gossip_app_get_window (),
+						   GOSSIP_STOCK_PENDING,
+						   GTK_ICON_SIZE_MENU);
 	}
 
 	return gossip_pixbuf_offline ();
@@ -796,7 +404,8 @@ gossip_pixbuf_for_chatroom_status (GossipChatroom *chatroom,
 		stock_id = GOSSIP_STOCK_GROUP_MESSAGE;
 		break;
 	case GOSSIP_CHATROOM_STATUS_ERROR:
-		stock_id = GTK_STOCK_DIALOG_ERROR;
+		/* NOTE: GTK_STOCK_DIALOG_ERROR is too big for menu images */
+		stock_id = GTK_STOCK_DIALOG_QUESTION;
 		break;
 	default:
 	case GOSSIP_CHATROOM_STATUS_INACTIVE:
@@ -805,207 +414,12 @@ gossip_pixbuf_for_chatroom_status (GossipChatroom *chatroom,
 		break;
 	}
 
-	pixbuf = gossip_pixbuf_from_stock (stock_id, icon_size);
+	pixbuf = gossip_stock_create_pixbuf (gossip_app_get_window (),
+					     stock_id, icon_size);
 
 	return pixbuf;
 }
 
-GdkPixbuf *
-gossip_pixbuf_avatar_from_vcard (GossipVCard *vcard)
-{
-	GdkPixbuf	*pixbuf;
-	GdkPixbufLoader	*loader;
-	const guchar    *avatar;
-	gsize		 len;
-	GError          *error = NULL;
-
-	g_return_val_if_fail (GOSSIP_IS_VCARD (vcard), NULL);
-
-	avatar = gossip_vcard_get_avatar (vcard, &len);
-	if (!avatar) {
-		return NULL;
-	}
-
-	loader = gdk_pixbuf_loader_new ();
-
-	if (!gdk_pixbuf_loader_write (loader, avatar, len, &error)) {
-		g_warning ("Couldn't write avatar image:%p with "
-			   "length:%" G_GSIZE_FORMAT " to pixbuf loader: %s",
-			   avatar, len, error->message);
-		g_error_free (error);
-		return NULL;
-	}
-
-	gdk_pixbuf_loader_close (loader, NULL);
-
-	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
-
-	g_object_ref (pixbuf);
-	g_object_unref (loader);
-
-	return pixbuf;
-}
-
-GdkPixbuf *
-gossip_pixbuf_avatar_from_contact (GossipContact *contact)
-{
-	GdkPixbuf	*pixbuf;
-	GdkPixbufLoader	*loader;
-	const guchar	*avatar;
-	gsize		 len;
-	GError          *error = NULL;
-
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
-
-	avatar = gossip_contact_get_avatar (contact, &len);
-	if (!avatar) {
-		return NULL;
-	}
-
-	loader = gdk_pixbuf_loader_new ();
-
-	if (!gdk_pixbuf_loader_write (loader, avatar, len, &error)) {
-		g_warning ("Couldn't write avatar image:%p with "
-			   "length:%" G_GSIZE_FORMAT " to pixbuf loader: %s",
-			   avatar, len, error->message);
-		g_error_free (error);
-		return NULL;
-	}
-
-	gdk_pixbuf_loader_close (loader, NULL);
-
-	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
-
-	g_object_ref (pixbuf);
-	g_object_unref (loader);
-
-	return pixbuf;
-}
-
-static void
-pixbuf_from_avatar_size_prepared_cb (GdkPixbufLoader *loader,
-				     int              width,
-				     int              height,
-				     struct SizeData *data)
-{
-	g_return_if_fail (width > 0 && height > 0);
-
-	if (data->preserve_aspect_ratio && (data->width > 0 || data->height > 0)) {
-		if (width < data->width && height < data->height) {
-			width = width;
-			height = height;
-		}
-
-		if (data->width < 0) {
-			width = width * (double) data->height / (gdouble) height;
-			height = data->height;
-		} else if (data->height < 0) {
-			height = height * (double) data->width / (double) width;
-			width = data->width;
-		} else if ((double) height * (double) data->width >
-			   (double) width * (double) data->height) {
-			width = 0.5 + (double) width * (double) data->height / (double) height;
-			height = data->height;
-		} else {
-			height = 0.5 + (double) height * (double) data->width / (double) width;
-			width = data->width;
-		}
-	} else {
-		if (data->width > 0) {
-			width = data->width;
-		}
-
-		if (data->height > 0) {
-			height = data->height;
-		}
-	}
-
-	gdk_pixbuf_loader_set_size (loader, width, height);
-}
-
-GdkPixbuf *
-gossip_pixbuf_from_avatar_scaled (const guchar *avatar,
-				  gsize         len,
-				  gint          width,
-				  gint          height)
-{
-	GdkPixbuf        *pixbuf;
-	GdkPixbufLoader	 *loader;
-	struct SizeData   data;
-	GError           *error = NULL;
-
-	if (!avatar) {
-		return NULL;
-	}
-
-	data.width = width;
-	data.height = height;
-	data.preserve_aspect_ratio = TRUE;
-
-	loader = gdk_pixbuf_loader_new ();
-
-	g_signal_connect (loader, "size-prepared",
-			  G_CALLBACK (pixbuf_from_avatar_size_prepared_cb),
-			  &data);
-
-	if (!gdk_pixbuf_loader_write (loader, avatar, len, &error)) {
-		g_warning ("Couldn't write avatar image:%p with "
-			   "length:%" G_GSIZE_FORMAT " to pixbuf loader: %s",
-			   avatar, len, error->message);
-		g_error_free (error);
-		return NULL;
-	}
-
-	gdk_pixbuf_loader_close (loader, NULL);
-
-	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
-
-	g_object_ref (pixbuf);
-	g_object_unref (loader);
-
-	return pixbuf;
-}
-
-GdkPixbuf *
-gossip_pixbuf_avatar_from_contact_scaled (GossipContact *contact,
-					  gint           width,
-					  gint           height)
-{
-	const guchar *avatar;
-	gsize         len;
-
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
-
-	avatar = gossip_contact_get_avatar (contact, &len);
-	if (!avatar) {
-		return NULL;
-	}
-
-	return gossip_pixbuf_from_avatar_scaled (avatar, len, width, height);
-}
-
-GdkPixbuf *
-gossip_pixbuf_avatar_from_vcard_scaled (GossipVCard *vcard,
-					GtkIconSize  size)
-{
-	const guchar *avatar;
-	gsize	      len;
-	gint          width, height;
-
-	g_return_val_if_fail (GOSSIP_IS_VCARD (vcard), NULL);
-
-	avatar = gossip_vcard_get_avatar (vcard, &len);
-	if (!avatar) {
-		return NULL;
-	}
-
-	if (!gtk_icon_size_lookup (size, &width, &height)) {
-		height = width = 48;
-	}
-
-	return gossip_pixbuf_from_avatar_scaled (avatar, len, width, height);
-}
-
 /* Stolen from GtkSourceView, hence the weird intendation. Please keep it like
  * that to make it easier to apply changes from the original code.
  */
@@ -1422,6 +836,252 @@ gossip_text_iter_forward_search (const GtkTextIter   *iter,
 	return retval;
 }
 
+static const gchar *
+g_utf8_strrcasestr (const gchar *haystack, const gchar *needle)
+{
+	gsize needle_len;
+	gsize haystack_len;
+	const gchar *ret = NULL;
+	gchar *p;
+	gchar *casefold;
+	gchar *caseless_haystack;
+	gint i;
+
+	g_return_val_if_fail (haystack != NULL, NULL);
+	g_return_val_if_fail (needle != NULL, NULL);
+
+	casefold = g_utf8_casefold (haystack, -1);
+	caseless_haystack = g_utf8_normalize (casefold, -1, G_NORMALIZE_NFD);
+	g_free (casefold);
+
+	needle_len = g_utf8_strlen (needle, -1);
+	haystack_len = g_utf8_strlen (caseless_haystack, -1);
+
+	if (needle_len == 0)
+	{
+		ret = (gchar *)haystack;
+		goto finally_1;
+	}
+
+	if (haystack_len < needle_len)
+	{
+		ret = NULL;
+		goto finally_1;
+	}
+
+	i = haystack_len - needle_len;
+	p = g_utf8_offset_to_pointer (caseless_haystack, i);
+	needle_len = strlen (needle);
+
+	while (p >= caseless_haystack)
+	{
+		if (strncmp (p, needle, needle_len) == 0)
+		{
+			ret = pointer_from_offset_skipping_decomp (haystack, i);
+			goto finally_1;
+		}
+
+		p = g_utf8_prev_char (p);
+		i--;
+	}
+
+finally_1:
+	g_free (caseless_haystack);
+
+	return ret;
+}
+
+static gboolean
+backward_lines_match (const GtkTextIter *start,
+		      const gchar      **lines,
+		      gboolean           visible_only,
+		      gboolean           slice,
+		      GtkTextIter       *match_start,
+		      GtkTextIter       *match_end)
+{
+	GtkTextIter line, next;
+	gchar *line_text;
+	const gchar *found;
+	gint offset;
+
+	if (*lines == NULL || **lines == '\0')
+	{
+		if (match_start)
+			*match_start = *start;
+		if (match_end)
+			*match_end = *start;
+		return TRUE;
+	}
+
+	line = next = *start;
+	if (gtk_text_iter_get_line_offset (&next) == 0)
+	{
+		if (!gtk_text_iter_backward_line (&next))
+			return FALSE;
+	}
+	else
+		gtk_text_iter_set_line_offset (&next, 0);
+
+	if (slice)
+	{
+		if (visible_only)
+			line_text = gtk_text_iter_get_visible_slice (&next, &line);
+		else
+			line_text = gtk_text_iter_get_slice (&next, &line);
+	}
+	else
+	{
+		if (visible_only)
+			line_text = gtk_text_iter_get_visible_text (&next, &line);
+		else
+			line_text = gtk_text_iter_get_text (&next, &line);
+	}
+
+	if (match_start) /* if this is the first line we're matching */
+	{
+		found = g_utf8_strrcasestr (line_text, *lines);
+	}
+	else
+	{
+		/* If it's not the first line, we have to match from the
+		 * start of the line.
+		 */
+		if (g_utf8_caselessnmatch (line_text, *lines, strlen (line_text),
+					   strlen (*lines)))
+			found = line_text;
+		else
+			found = NULL;
+	}
+
+	if (found == NULL)
+	{
+		g_free (line_text);
+		return FALSE;
+	}
+
+	/* Get offset to start of search string */
+	offset = g_utf8_strlen (line_text, found - line_text);
+
+	forward_chars_with_skipping (&next, offset, visible_only, !slice, FALSE);
+
+	/* If match start needs to be returned, set it to the
+	 * start of the search string.
+	 */
+	if (match_start)
+	{
+		*match_start = next;
+	}
+
+	/* Go to end of search string */
+	forward_chars_with_skipping (&next, g_utf8_strlen (*lines, -1), visible_only, !slice, TRUE);
+
+	g_free (line_text);
+
+	++lines;
+
+	if (match_end)
+		*match_end = next;
+
+	/* try to match the rest of the lines forward, passing NULL
+	 * for match_start so lines_match will try to match the entire
+	 * line */
+	return lines_match (&next, lines, visible_only,
+			    slice, NULL, match_end);
+}
+
+gboolean
+gossip_text_iter_backward_search (const GtkTextIter   *iter,
+				  const gchar         *str,
+				  GtkTextIter         *match_start,
+				  GtkTextIter         *match_end,
+				  const GtkTextIter   *limit)
+{
+	gchar **lines = NULL;
+	GtkTextIter match;
+	gboolean retval = FALSE;
+	GtkTextIter search;
+	gboolean visible_only;
+	gboolean slice;
+
+	g_return_val_if_fail (iter != NULL, FALSE);
+	g_return_val_if_fail (str != NULL, FALSE);
+
+	if (limit && gtk_text_iter_compare (iter, limit) <= 0)
+		return FALSE;
+
+	if (*str == '\0')
+	{
+		/* If we can move one char, return the empty string there */
+		match = *iter;
+
+		if (gtk_text_iter_backward_char (&match))
+		{
+			if (limit && gtk_text_iter_equal (&match, limit))
+				return FALSE;
+
+			if (match_start)
+				*match_start = match;
+			if (match_end)
+				*match_end = match;
+			return TRUE;
+		}
+		else
+		{
+			return FALSE;
+		}
+	}
+
+	visible_only = TRUE;
+	slice = TRUE;
+
+	/* locate all lines */
+	lines = strbreakup (str, "\n", -1);
+
+	search = *iter;
+
+	while (TRUE)
+	{
+		/* This loop has an inefficient worst-case, where
+		 * gtk_text_iter_get_text () is called repeatedly on
+		 * a single line.
+		 */
+		GtkTextIter end;
+
+		if (limit && gtk_text_iter_compare (&search, limit) <= 0)
+			break;
+
+		if (backward_lines_match (&search, (const gchar**)lines,
+					  visible_only, slice, &match, &end))
+		{
+			if (limit == NULL || (limit &&
+					      gtk_text_iter_compare (&end, limit) > 0))
+			{
+				retval = TRUE;
+
+				if (match_start)
+					*match_start = match;
+				if (match_end)
+					*match_end = end;
+			}
+			break;
+		}
+
+		if (gtk_text_iter_get_line_offset (&search) == 0)
+		{
+			if (!gtk_text_iter_backward_line (&search))
+				break;
+		}
+		else
+		{
+			gtk_text_iter_set_line_offset (&search, 0);
+		}
+	}
+
+	g_strfreev ((gchar**)lines);
+
+	return retval;
+}
+
 /* The URL opening code can't handle schemeless strings, so we try to be
  * smart and add http if there is no scheme or doesn't look like a mail
  * address. This should work in most cases, and let us click on strings
@@ -1575,3 +1235,16 @@ gossip_request_user_attention (void)
 #endif
 }
 
+
+void
+gossip_toggle_button_set_state_quietly (GtkWidget *widget,
+					GCallback  callback,
+					gpointer   user_data,
+					gboolean   active)
+{
+	g_return_if_fail (GTK_IS_TOGGLE_BUTTON (widget));
+
+	g_signal_handlers_block_by_func (widget, callback, user_data);
+	g_object_set (widget, "active", active, NULL);
+	g_signal_handlers_unblock_by_func (widget, callback, user_data);
+}
